//
// stat.c
//
// a program to test out the pertelian API.  Sends hostname, cpu speed,
// uptime, load averages, date and time to the display unit.  Currently
// only works in linux, needs some #ifdef foo.
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <linux/sysctl.h>
#include <sys/sysinfo.h>
#include "pertelian.h"

// how often should we update the uptime?
#define UPTIMEDELAY 60

char * getlaverage(void);     // prepare string with the three load averages
void getmemories(void);       // grab memory data from /proc/meminfo
void punt (char *, int);      // general error/exit routine
void cleanup (void);          // atexit(3) handler to shut down the device on
                              // exit (this program won't exit normally)
void sigexit(int);            // signal handler

static DEVICE *device = NULL;
static FILE *meminfo = NULL;

char memline1[21], memline2[21], memline3[21];

int main (int ac, char **av)
//
// main routine
//
// arguments:  unused at this time
//
// return value:  zero on successful completion.  but since there's an 
//                infinite loop in the code, it doesn't complete.  Exits
//                happen only on caught signals 2, 3, 15
//
{
  atexit(cleanup);              // have the program gently close
                                // the door on the way out.
  signal (SIGINT, sigexit);     // catch various signals to shut down 
  signal (SIGQUIT, sigexit);    // the unit.
  signal (SIGTERM, sigexit);

    // open the device.  maybe we should dig around in the
    // /dev directory looking for it, since we won't know the
    // name of the device.  Not worth the trouble for a test
    // program, so we'll leave it up to the user to make the correct
    // symlink here

  if ((device = fopen ("/dev/ttyUSB0", "w")) == NULL) {
    punt ("cannot fopen gizmo", errno);
  }

    // initialize the device per pertelian's recommendation
    // (can't talk to it otherwise)

  if (init (device) == EOF) {
    punt ("cannot write gizmo (init)", errno);
  }

    // clear the screen (in case someone else was using the thing)

  if (clearscreen (device) == EOF) {
    punt ("cannot clear gizmo (clearscreen)", errno);
  }

    // turn on the light
  if (backlight (BACKLIGHT_ON, device) == EOF) {
    punt ("cannot start backlight (backlight)", errno);
  }

  // OK, we should now be ready, finally, to send some real
  // data to the unit!

  for (;;)   // we're going to loop until we catch a signal
  {
    if (writeline (device, getlaverage(), 1, 1) == EOF) {
      punt ("can't write hostname (writeline)", errno);
    }

    getmemories();

    if (writeline (device, memline1, 1, 2) == EOF) {
      punt ("can't write uptime (writeline)", errno);
    }

    if (writeline (device, memline2, 1, 3) == EOF) {
      punt ("can't write laverage (writeline)", errno);
    }

    if (writeline (device, memline3, 1, 4) == EOF) {
      punt ("can't write date/time (writeline)", errno);
    }

    sleep(10);  // don't forget to breathe
  }

  return 0;  // never executed
}


char * getlaverage(void)
//
// format the load average into something sensible
// we don't have a lot of room for high load averages, 
// so we're going to do a variable number of decimals
// so that each lav will take no more than 4 spaces
// on the display
//
// arguments:  none
//
// return value:  static storage of the formatted string
{
  struct sysinfo info;
  float lav[3];
  char lavstr[3][5];
  int i;
  static char outstr[21];

    // laverage includes three integers and a single integer
    // scaling factor you can use as a denominator
    // so, for each of the three values, divide by the scaling 
    // factor, then convert the resulting float into a string
    // in a 4-character (pluss null) format

  if (sysinfo(&info) != -1) {
    for (i = 0; i <= 2; i++) {
      lav[i] = (float) info.loads[i] / 65536.0;
      if (lav[i] > 100.0)     sprintf (lavstr[i], "%4.0f", lav[i]);
      else if (lav[i] > 10.0) sprintf (lavstr[i], "%4.1f", lav[i]);
      else                    sprintf (lavstr[i], "%4.2f", lav[i]);
    }
      // format the output
    snprintf (outstr, 21, "lav:  %s %s %s",
              lavstr[0], lavstr[1], lavstr[2]);
  } else
     punt ("can't get load averages", errno);

  return outstr;  // share with the world
}


void getmemories(void)
{
  int memtotal, memfree, memused, buffers, cached;
  char *temp;
  char buf[256];

  if ((meminfo = fopen ("/proc/meminfo", "r")) == NULL)
    punt ("can't open meminfo", errno);

  if (meminfo) {
    fgets(buf, 256, meminfo);
    temp = strstr(buf, ":") + 1;
    sscanf(temp, "%d", &memtotal);
    fgets(buf, 256, meminfo);
    temp = strstr(buf, ":") + 1;
    sscanf(temp, "%d", &memfree);
    fgets(buf, 256, meminfo);
    temp = strstr(buf, ":") + 1;
    sscanf(temp, "%d", &buffers);
    fgets(buf, 256, meminfo);
    temp = strstr(buf, ":") + 1;
    sscanf(temp, "%d", &cached);
    fclose(meminfo);
    meminfo = NULL;
  }
  memused = memtotal - memfree;
  snprintf(memline1, 21, "use: %7d buf/cch", memused);
  snprintf(memline2, 21, "fre: %7d %7d", memfree, buffers);
  snprintf(memline3, 21, "tot: %7d %7d", memtotal, cached);
}

void punt (char *errmsg, int retval)
//
// quick and dirty error handler
//
// arguments:  errmsg:  null terminated error message (does not
//                      require a trailing newline, punt will add
//                      one for you
//             retval:  whatever value you want to pass back to the
//                      shell.  errno is often a good choice if
//                      you don't have any ideas of your own.
//
// return value:  none
{
  fprintf (stderr, "%s (%d)\n", errmsg, retval);
  exit (retval);
}

void cleanup(void)
// 
// clear screen and shut down unit.  called via atexit(3)
//
// arguments:  none, requires the device handle be a global.
//
// return value:  none
//
{
  if (device) {
    clearscreen(device);               // clear the data
    backlight(BACKLIGHT_OFF, device);  // turn off the lights
    if (meminfo) fclose (meminfo);     // take out the trash
    fclose (device);                   // close the door
  }
}

void sigexit(int sig)
//
// handle caught signal.  we'll just call exit(3) here and let 
// the atexit cleanup function take care of the rest
//
// Arguments:  sig:  the caught signal.  Ignored.
//
// Return value:  none
//
{
  exit(0);
}
