//
// pertelian.c
//
// a simple API to address the Pertelian X2040 LCD display
//
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include "pertelian.h"

int init (DEVICE *d)
//
// initialize the device to take input.  You probably don't want to
// experiment too much with this.  See the Hitachi docs.
//
// Arguments:  device handle
// Return value:  Non-zero on successful completion.  Sort of.
//
{
  int r = 0;

  if (d) {
    command (0x38, d);         // 8-bit data, 2 lines (yes, really)
                               // and 5x7 character cells
    command (0x06, d);         // Set cursor to move left-to-right, no
                               // text shifting
    command (0x10, d);         // cursor moves with text
    r = command (0x0c, d);     // Display on, cursor off, blink off
                               // we return the result so as to
                               // return *something*.  Probably not
  }                            // useful...

  return r;
}


int clearscreen (DEVICE *d)
//
// clear the display and home the cursor
//
// Arguments:  device handle
// Return value:  Non-zero on successful completion.
//
{
  int r = 0;

  if (d) {
    r = command(1, d);  // clear display, home cursor
    usleep (10000);     // wait 10 ms for the dust to settle
  } 

  return r;
}


int backlight (int state, DEVICE *d)
//
// set the state of the backlight.  this is a pertelian-specific
// feature
//
// Arguments:  state: BACKLIGHT_OFF or BACKLIGHT_ON defined in 
//             pertelian.h (BACKLIGHT_ON can be any nonzero value;
//             d: device handle
//
// Return value:  nonzero on successful completion
//
{
  int r = 0;

  if (d)
    r = command ((2 | (state ? 1 : 0)), d);   // send the command

  return r;
}


int writeline (DEVICE *d, char *buf, int col, int row)
//
// write characters to the display starting at a specified row and 
// column.  rows and columns start at (1,1).  other than a quick sanity
// check on the row and column number, no bounds checking 
// is done; it's considered a feature to be able to write past the
// end of the line.
//
// Arguments:  d:  device handle
//             buf:  null-terminated string to be output
//             col:  column number of first character written
//             row:  row number of first character written
//
// return value:  number of characters written, or EOF on failure.
//                errno is ERANGE if row or col are out of bounds,
//                or as per fputs(3)
//
{
  int r = 0;

  if (d) {
      // precalculate pointer positions.  rowvals will hold the
      // DDRAM addresses of each row at the given column.
      // (not really a linear relation between row/col and 
      // memory location, alas)
    const unsigned int rowvals[5] = { 0, (0x80 + (col - 1)),
                             (0x80 + (col - 1) + 0x40),
                             (0x80 + (col - 1) + 0x14),
                             (0x80 + (col - 1) + 0x54) };
  
    if ((row < 1) || (row > 4)) {     // check for correct row number
      errno = ERANGE;
      return EOF;
    }
  
    if ((col < 1) || (col > 0x14)) {  // check for correct column number
      errno = ERANGE;
      return EOF;
    }
  
    command (rowvals[row], d);        // set cursor, ready for ascii data
  
    r = slowfputs(buf, d);         // send ascii data and return number
  }                                    // of chars sent

  return r;
}


int command (unsigned char c, DEVICE *d)
//
// send command (as opposed to data) to the device.  This is pertelian-
// specific.  The Hitachi chip as a data/instruction hardware line. 
// the pertelian overprocessor interprets a specific character input
// (COMMAND, defined in pertelian.h) as an instruction to set the 
// instruction line.  All Hitachi instructions are one byte in length,
// so the overprocessor resets the instruction line after sending the
// COMMAND character.  Ergo, we prefix every command sent with COMMAND.
//
// Arguments:  c:  command byte
//             d:  device handle
//
// Return value:  nonzero on success
{
  int r = 0;

  if (d) {
    char buf[3] = {COMMAND, 0, 0};  // Assemble the command buffer.  both
    buf[1] = c;                     // characters.
    r = slowfputs (buf, d);         // Write to the device
    usleep(1640);                   // most commands are guaranteed to be
                                    // completed in 1.64 ms
  }

  return r;                         // return number of chars sent
}

int slowfputs(char *buf, DEVICE *d)
//
// a rewrite of fputs(3) to incorporate a delay loop.  This is a hack.
// if we really had it, we would check the status of a busy line on the 
// Hitachi chip, but pertelian seems not to have provided a way to 
// check it.  Currently we're waiting 1 ms after each character sent;
// hopefully, we can lower that in the future.
//
// Arguments:  buf:  null-terminated string to be sent to the device
//             d:    device handle
//
// Return value:  number of characters sent, or EOF on failure
//
{
  int r;
  int count = 0;
  
  if (d) {
    while (*buf) {            // for each character in buf
      r = putc (*buf, d);     // try to send it with putc(3).
      if (r != EOF) count++;  // count the character if successful
      else return EOF;        // and punt if not.
      fflush (d);             // make sure the byte gets out the door.
      buf++;                  // move on to the next character
      usleep(1000);           // and don't forget to breathe.
    }
  }

  return count;             // if we made it here, it must have  
                            // all worked out.  return char count.
}

